@@ -25,7 +25,6 @@ class ModelRouterConfig(BaseModel):
 
 
 class ModelDeploymentConfig(BaseModel):
-    deployment: str
     model: str
     served_model_name: str
     batch_size: Optional[int] = 8
@@ -35,10 +34,13 @@ class ModelDeploymentConfig(BaseModel):
     matryoshka_dim: Optional[int] = 768
     trust_remote_code: Optional[bool] = False
     model_kwargs: Optional[Dict[str, Any]] = {}
+    cuda_memory_flush_threshold: Optional[float] = 0.8
+    deployment: str
 
 
 class AppConfig(BaseModel):
     model_router: ModelRouterConfig
+    node_health_check_interval_s: Optional[int] = 30
     models: List[ModelDeploymentConfig]
 
 
